﻿#ifndef 判定_H_
#define 判定_H_

#include <string>
#include <algorithm>
#include "エラー定数.h"

/*! \file 判定.h
    \sk はんてい.h
	\brief ヒットアンドブローの判定結果を受け渡すための構造体や関数の宣言

\sa 判定.cpp
*/

/*! \brief ヒットアンドブローの判定結果を保持する構造体
    \sk はんていけっか

関数 判定() が返す値の型です。
*/
struct 判定結果
{
	/*! \brief 複製構築子
	
	\param rhs 複製元
	*/
	判定結果( const 判定結果& rhs ) : mHit( rhs.mHit ), mBlow( rhs.mBlow ) { }

	/*! \brief 一般構築子

	\param hit 位置まで含めて一致した数
	\param blow 位置は異なるが利用している要素ではある数、hitの分は含めません
	*/
	判定結果( const int hit, const int blow ) : mHit( hit ), mBlow( blow ) { }
	const int 計() const { return mHit + mBlow; } ///< \sk けい \return ヒットとブローの合計を返します
	const int mHit; ///< \sk ひっと 位置まで含めて一致した数
	const int mBlow; ///< \sk ぶろー 位置は異なるが要素ではある数、 mHit は含まない

	static エラー定数 m最終エラー; ///< \sk さいしゅうえらー 最新の判定に対する前提条件の成否を示します
	/*! \brief 最新判定が有効であるかどうかを返します
	    \sk はんていせいこう

	\return 最新判定の前提条件に問題がなければ真を返します
	*/
	static bool 判定成功() { return m最終エラー == エラーなし; }
};

/*! \brief ヒットアンドブローの判定を行います

内部処理としては 解答 と 試行 の両引数に処理上の区別はありません。

\tparam T 逐次反復子と string や vector のような [] 演算子が使えるクラスを指定ください
\param 解答 隠されている正解を与えてください
\param 試行 挑戦者側の予想を与えてください
\return 判定結果を返します、引数が正常かどうかを調べるために 判定結果::判定成功() の戻り値も調べてください
*/
template <class T>
判定結果 判定( const T& 解答, const T& 試行 )
{
	if ( 解答.size() != 試行.size() )
	{
		std::wcerr << L"解答と試行の長さが違います。" << std::endl;
		判定結果::m最終エラー = 判定長さの不一致;
		return 判定結果( 0, 0 );
	}
	int hit = 0;
	int blow = 0;
	int idx = 0;
	for ( auto it = 解答.begin(); it != 解答.end(); ++it, ++idx )
	{
		const auto 発見位置 = std::find( 試行.begin(), 試行.end(), *it );
		if ( 発見位置 != 試行.end() )
		{
			if ( *発見位置 == *(試行.begin() + idx) )
			{
				hit++;
			}
			else
			{
				blow++;
			}
		}
	}
	return 判定結果( hit, blow );
}
#endif
