﻿#include <iostream>
#include "推定.h"

/*! \file 推定.cpp
    \brief 推定に関する処理の定義が含まれています。

…というふうに 推定.cpp に対する文書も書いておかないと
関数の定義詳解がないものと扱われてしまいます。
*/

/*! \brief 構築子

与える引数は 順列 や 組み合わせ の各クラスと同じです。

\param 元数 各桁が取りうる値の種類数です、各桁には 0 - (元数-1) のいずれかが入ります
\param 取得長さ 返される文字数はこの値と一致します
*/
推定::推定( const unsigned int 元数, const unsigned int 取得長さ)
	: m元数( 元数 ), m取得長さ( 取得長さ ? 取得長さ : 元数 ),
	mヒット探索( 0 ), mブロー探索( 0 ), m段階( 利用要素探索 )
{
	mブロー探索 = 組み合わせ( m元数, m取得長さ );
	mヒット探索 = 順列( m取得長さ ); // 使われるのは全ブローが成立した後なので元数は使わない
}


/*! \brief 次の試行値を配列の形で返します

返した配列を解答と比較し、完全一致でなければさらに 結果登録() 関数で登録してください。

\return 次の試行値です
*/
std::vector<unsigned int> 推定::次()
{
	switch ( m段階 )
	{
	case 利用要素探索:
		return mブロー探索.次();
	case 並べ替え:
	{
		const auto v = mヒット探索.次();
		std::vector<unsigned int> r;
		for ( auto it = v.begin(); it != v.end(); ++it )
		{
			r.push_back( m全ブロー[ *it ] );
		}
		return r;
	}
	default:
		return std::vector<unsigned int>();
	}
}


/*! \brief 判定結果を登録します \sk けっかとうろく

現実装では全桁がヒットまたはブローになったときだけ意味を持っています。

\param 試行 次() 関数の返した値をご指定ください
\param 結果 解答との比較判定結果をご指定ください
*/
void 推定::結果登録( const std::vector<unsigned int>& 試行, const 判定結果& 結果 )
{
	switch ( m段階 )
	{
	case 利用要素探索:
		if ( 結果.計() == m取得長さ )
		{
			m段階 = 並べ替え;
			m全ブロー = 試行;
			std::wcout << std::endl << L"使われている文字を特定しました!" << std::endl;
		}
		return;
	case 並べ替え:
		return;
	default:
		return;
	}
}
