﻿#ifndef 推定_H_
#define 推定_H_

#include <map>
#include "判定.h"
#include "順列.h"

/*! \file 推定.h
    \sk すいてい.h
	\brief ヒットアンドブローの正解を予測するためのクラスを宣言しています

*/

/*! \brief ヒットアンドブローの正解を予測するための情報蓄積および分析を行います

*/
class 推定
{
public:
	/*! \brief 推定段階の遷移状態を示します \sk すいてい

	論理的には内部利用のみで完結しています。

	元実装では総当たりですべての桁がヒットまたはブローしたところで
	順列検索に移行するようにしか作っていません。
	*/
	enum 段階
	{
		利用要素探索, ///< \sk りようようそたんさく すべてをブローさせるまで探す
		並べ替え, ///< \sk ならべかえ 全ブローから全ヒットへ進行中
		エラー発生, ///< \sk えらーはっせい 使われていません
	};
	推定( const unsigned int 元数, const unsigned int 取得長さ = 0 );
	std::vector<unsigned int> 次();
	void 結果登録( const std::vector<unsigned int>& 試行, const 判定結果& 結果 );
protected:
	std::map<std::wstring, 判定結果> m応答蓄積; ///< \sk おうとうちくせき 使われていません
	組み合わせ mブロー探索; ///< \sk ぶろーたんさく 最初の総当たりで使います
	順列 mヒット探索; ///< \sk ひっとたんさく 構成要素特定後の順列総当たりで使います
	段階 m段階; ///< \sk だんかい 現在の検索処理段階を示します
	const unsigned int m元数; ///< mブロー探索 に与えた初期値と同じです
	const unsigned int m取得長さ; ///< mブロー探索 や mヒット探索 に与えた初期値と同じです
	std::vector<unsigned int> m全ブロー; ///< \sk ぜんぶろー 利用要素が確定したときに代入します
};

#endif
