﻿#ifndef 順列_H_
#define 順列_H_

#include <vector>

/*! \file 順列.h
    \sk じゅんれつ.h
	\brief 順列または組み合わせの反復取得クラスを宣言しているヘッダファイルです

\sa 順列.cpp
*/

/*! \brief 指定された引数に基づく順列を逐次生成して返します

構築子で与えた元の数と返すべき集合の大きさを満たす順列を
std::vector<unsigned int> の形で関数 次() が呼び出されるたびに返します。

doxygen の動作試験用に作っただけのため簡素な設計になっています。
*/
class 順列
{
public:
	順列( const unsigned int 元数, const unsigned int 取得長さ = 0 );
	/*! \brief 複製構築子

	全変数を複製します。処理中であった場合は処理中であるその状態もそのまま複製します。
	\param rhs 複製元
	*/
	順列( const 順列& rhs ) : m元数( rhs.m元数 ), m取得長さ( rhs.m取得長さ ) { }
	/*! \brief 代入演算子

	全変数を複製します。処理中であった場合は処理中であるその状態もそのまま複製します。
	\param rhs 複製元
	\return 値を新しく代入された自分自身
	*/
	順列& operator = ( const 順列& rhs )
	{
		m元数 = rhs.m元数;
		m取得長さ = rhs.m取得長さ;
		return *this;
	}
	virtual ~順列() { } ///< \sk ~じゅんれつ 何もしていません
	void 初期化() { m現況.clear(); } ///< \sk しょきか 次() が返す配列を最初に巻き戻します
	std::vector<unsigned int> 次();
	unsigned int 総数() const;
	void 全表示();

public:
	unsigned int m元数; ///< \sk げんすう 要素の種類数です
	unsigned int m取得長さ; ///< \sk しゅとくながさ 毎回返す集合の大きさです
protected:
	std::vector<unsigned int> m現況; ///< \sk げんきょう 直前に返した組を保持しています
};


/*! \brief 指定された引数に基づく組み合わせを逐次生成して返します
    \sk くみあわせ

構築子で与えた元の数と返すべき集合の大きさを満たす組み合わせを
std::vector<unsigned int> の形で関数 次() が呼び出されるたびに返します。

順列 と同じく doxygen 動作試験用の簡素な設計になっています。
*/
class 組み合わせ
{
public:
	組み合わせ( const unsigned int 元数, const unsigned int 取得長さ = 0 );
	/*! \brief 複製構築子

	全変数を複製します。処理中であった場合は処理中であるその状態もそのまま複製します。

	\param rhs 複製元
	*/
	組み合わせ( const 組み合わせ& rhs ) : m順列( rhs.m順列 ) { }
	/*! \brief 代入演算子

	全変数を複製します。処理中であった場合は処理中であるその状態もそのまま複製します。
	\param rhs 複製元
	\return 値を新しく代入された自分自身
	*/

	組み合わせ& operator = ( const 組み合わせ& rhs )
	{
		m順列 = rhs.m順列;
		return *this;
	}
	virtual ~組み合わせ() { } ///< \sk ~くみあわせ 何もしていません
	void 初期化() { m順列.初期化(); } ///< 次() が返す配列を最初に巻き戻します
	std::vector<unsigned int> 次();
	unsigned int 総数() const;
	void 全表示();

protected:
	順列 m順列; ///< \sk じゅんれつ 出力する組み合わせの元になる順列
};

#endif
